import { ɵdetectChanges, ɵrenderComponent, ViewEncapsulation } from '@angular/core';

function autoChangeDetection(component, inputs, props) {
  const keys = Object.keys(inputs);
  keys.forEach((key) => {
    props.set(key, component[key]);
    Object.defineProperty(component, key, {
      get() {
        return props.get(key);
      },
      set(value) {
        props.set(key, value);
        ɵdetectChanges(component);
      }
    });
  });
  return component;
}
function initProps(target, inputs, component) {
  const keys = Object.keys(inputs);
  keys.forEach((key) => {
    Object.defineProperty(target, key, {
      get() {
        return component[key];
      },
      set(value) {
        component[key] = value;
      }
    });
  });
}
function initEvents(target, outputs, component) {
  const outputEvents = Object.keys(outputs);
  outputEvents.forEach((outputEvent) => {
    component[outputs[outputEvent]].subscribe((info) => {
      target.dispatchEvent(new CustomEvent(outputEvent, {detail: info}));
    });
  });
}

function getDirectiveDef(t) {
  if (t["ɵdir"]) {
    return t["ɵdir"];
  }
  if (t["ɵcmp"]) {
    return t["ɵcmp"];
  }
  throw new Error("No Angular definition found for " + t.name);
}
function getPipeDef(t) {
  if (t["ɵpipe"]) {
    return t["ɵpipe"];
  }
  throw new Error("No Angular definition found for " + t.name);
}
function updateComponentDef(componentType, options) {
  const def = componentType["ɵcmp"];
  def.directiveDefs = (options == null ? void 0 : options.directives) ? options.directives.filter((t) => t["ɵdir"]).map((directive) => getDirectiveDef(directive)) : null;
  def.pipeDefs = (options == null ? void 0 : options.pipes) ? options.pipes.map((pipe) => getPipeDef(pipe)) : null;
  return def;
}

function renderComponentDirective(types, options) {
  const directives = options.directives.filter((t) => t["ɵdir"]);
  const components = options.directives.filter((t) => t["ɵcmp"]);
  components.forEach((type) => {
    renderCustomElement(type, {...options, directives});
  });
}
function renderNgComponent(componentType, options) {
  const props = new Map();
  const def = updateComponentDef(componentType, options);
  const component = ɵrenderComponent(componentType, {...options != null ? options : {}, rendererFactory: null});
  return autoChangeDetection(component, def.inputs, props);
}
function createCustomElement(componentType, options) {
  const {inputs, styles, encapsulation, outputs} = componentType["ɵcmp"];
  (options == null ? void 0 : options.directives) && renderComponentDirective(options.directives, options);
  return class CustomElement extends HTMLElement {
    constructor() {
      super();
      this.rootElement = encapsulation === ViewEncapsulation.ShadowDom ? this.attachShadow({mode: "open"}) : this;
      this.component = renderNgComponent(componentType, {
        ...options != null ? options : {},
        host: this.rootElement
      });
      initProps(this, inputs, this.component);
      initEvents(this, outputs, this.component);
    }
    static get observedAttributes() {
      return [...Object.keys(inputs)];
    }
    attributeChangedCallback(name, oldValue, newValue) {
      const attributes = Object.keys(inputs);
      if (attributes.includes(name)) {
        this[name] = newValue;
      }
    }
    connectedCallback() {
      const cssStyles = styles.join("\n");
      if (cssStyles) {
        const style = document.createElement("style");
        style.textContent = cssStyles;
        this.rootElement.prepend(style);
      }
    }
  };
}
function renderCustomElement(componentType, options) {
  customElements.define(componentType["ɵcmp"].selectors[0][0], createCustomElement(componentType, options));
}

export { createCustomElement, renderCustomElement, renderNgComponent };
